/*!
  * @license MIT (https://github.com/geotrev/core-flux/blob/master/LICENSE)
  * Core Flux v2.0.0 (https://github.com/geotrev/core-flux)
  * Copyright 2023 George Treviranus <geowtrev@gmail.com>
  */
function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);
  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    enumerableOnly && (symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    })), keys.push.apply(keys, symbols);
  }
  return keys;
}
function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = null != arguments[i] ? arguments[i] : {};
    i % 2 ? ownKeys(Object(source), !0).forEach(function (key) {
      _defineProperty(target, key, source[key]);
    }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) {
      Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
    });
  }
  return target;
}
function _defineProperty(obj, key, value) {
  key = _toPropertyKey(key);
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }
  return obj;
}
function _toPrimitive(input, hint) {
  if (typeof input !== "object" || input === null) return input;
  var prim = input[Symbol.toPrimitive];
  if (prim !== undefined) {
    var res = prim.call(input, hint || "default");
    if (typeof res !== "object") return res;
    throw new TypeError("@@toPrimitive must return a primitive value.");
  }
  return (hint === "string" ? String : Number)(input);
}
function _toPropertyKey(arg) {
  var key = _toPrimitive(arg, "string");
  return typeof key === "symbol" ? key : String(key);
}

var Stores = new Map();
var idValue = 0;
function getTypeTag(value) {
  return Object.prototype.toString.call(value).slice(8, -1).toLowerCase();
}
function isPlainObject(value) {
  return getTypeTag(value) === "object";
}
function throwError(msg) {
  throw new Error("[core-flux] ".concat(msg));
}
function createId() {
  return idValue + 1;
}
function setInitialStore(id, initialState) {
  Stores.set(id, {
    state: initialState,
    subscriptions: []
  });
}
function getState(id) {
  return Stores.get(id).state;
}
function setState(id, rawNextStateValue) {
  if (!isPlainObject(rawNextStateValue)) {
    return throwError("bindState callback: The reduced state value must be a plain object. If there is no change in state, simply return it.");
  }
  var store = Stores.get(id);
  if (!store) return;
  var state = store.state,
    subscriptions = store.subscriptions;
  var newState = _objectSpread2(_objectSpread2({}, state), rawNextStateValue);
  Stores.set(id, {
    state: newState,
    subscriptions: subscriptions
  });
}
function createStore(initialState, reducer, bindSubscriber, bindState) {
  if (!initialState || !isPlainObject(initialState)) {
    return throwError("createStore(): The initial state value must be a plain object.");
  }
  var id = createId();
  setInitialStore(id, initialState);
  return {
    dispatch: function dispatch(type, payload) {
      var state = getState(id);
      var reducedState = reducer(state, {
        type: type,
        payload: payload
      });
      return bindState(Stores.get(id).subscriptions, reducedState, function (rawNextState) {
        return setState(id, rawNextState);
      });
    },
    subscribe: function subscribe(subscriber, data) {
      if (!subscriber || !data) {
        return throwError("subscribe(): `subscriber` and `data` arguments are required.");
      }
      var _Stores$get = Stores.get(id),
        subscriptions = _Stores$get.subscriptions,
        state = _Stores$get.state;
      subscriptions.push([subscriber, data]);
      bindSubscriber(subscriptions[subscriptions.length - 1], state);
    },
    __data: Stores.get(id)
  };
}

export { createStore };
//# sourceMappingURL=core-flux.es.js.map
